/* logging.c
 */

/**
 * \page logging Data Logging
 * \todo IDs, eepromschreiben, dataflash schreiben
 * \todo uartkommunikation zum Auslesen
 * \todo x,z-modem Protokoll zur Datenausgabe?
 * data logging in internal eeprom or external serial flash
 *
 * Der AT45DB041B kann beim Programmieren bis zu 35mA ziehen!
 * => Main Memory Page Program without Erase, lschen tun wir mit einer extra Option 
 * => Erst beide Buffer vollschreiben, und dann programmieren?
 * 
 * 
 */

#include <stdio.h>
#include <inttypes.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include "main.h"
#include "settings.h"
#include "logging.h"
#include "ui.h"
#if LOG_TO_DATAFLASH
#include "dataflash.h"
#elif LOG_TO_EEPROM
#include <avr/eeprom.h>
#endif

extern volatile uint32_t totaldistance;
extern FILE *uart;
extern uint32_t systime;
extern volatile char key;
extern char username_long[]; 
extern char bikename[];

struct{
	int distpause:1;
}logflag;


uint16_t bufferadr;
uint16_t pageadr;

/**
 * \return -1 when memory full, otherwise 0
 */
int8_t program_page(void)
{
	if (pageadr>DF_NOOF_PAGES)
	{
		puts_P(PSTR("LogFul\n"));
		return -1;
	}
	buffer_to_flash_no_erase(1,pageadr);
	pageadr++;
	return 0;
}

/**
 * simple hex dump of dataflash content
 */
void log_hex_output(void)
{
	uint32_t i;
	uint8_t j=0;
	program_page();
	while(!(flash_status()&0x80))
			;
	cont_array_read_init();
	for(i=0;i<0x10001;i++)
	{
		if(key==KEY_PUSH)
			break;	
		fprintf_P(uart,PSTR("\n%.6lX\t"),(uint32_t)(i<<4));
		printf_P(PSTR("%6ld\n"),i*j);
		for(j=0;j<16;j++)
		{
			fprintf_P(uart,PSTR("%.2X "),df_spi_rw(0));
		}
	}
	df_cs_inactive;
	fputs("",uart);
}

/**
 * human readable logdata output
 */
void log_ascii_output(void)
{
	fputs_P(PSTR("USERNAME\t"),uart);
	fputs_P(username_long,uart);
	fputs_P(PSTR("\n"),uart);
	fputs_P(PSTR("BIKE\t"),uart);
	fputs_P(bikename,uart);
	fputs_P(PSTR("\n"),uart);
	fprintf_P(uart,PSTR("LOGSIZE\t%lu\n"),(uint32_t) pageadr * DF_PAGE_SIZE + bufferadr);
	fprintf_P(uart,PSTR("SYSTIME\t%lu\n"),systime);
}

/**
 * binary data output organized in 256 bit chunks
 * \todo XModem Data Transfer Protocol?
 */
void log_binary_output(void)
{

}

void write_word(uint16_t word)
{
	buffer_write_init(1,bufferadr);
	df_spi_rw((uint8_t) word);
	df_spi_rw((uint8_t) word>>8);
	df_cs_inactive;
	bufferadr+=2;
	if (bufferadr>DF_PAGE_SIZE)
		program_page();
}
/**
 * \return Percentage of used Log Memory
 */
uint8_t log_mem_used()
{
	uint32_t percent = pageadr*100;
	percent/=DF_NOOF_PAGES;
	return percent;
}

/**
 * delete all log entries
 * \todo only erase already programmed blocks
 */
void log_delete(void)
{
#if DEBUG
	fprintf(uart,"%s: Deleting Log Entries\n", __FUNCTION__);
#endif
	//cli();
	erase_all();
	bufferadr=0;
	pageadr=0;
	//sei();
#if DEBUG
	fprintf(uart,"%s: Log Entries Deleted\n", __FUNCTION__);
#endif
}

/**
 * wird periodisch im Abstand LOG_UNIT von der RTC-Routine aufgerufen
 */
void logupdate(void)
{
	static uint32_t lastdist;
	uint16_t distdiff;
#if DEBUG
	fprintf(uart,"%s\n",__FUNCTION__);
#endif
	distdiff = totaldistance - lastdist;
	if (distdiff)
	{
		lastdist = totaldistance;
		if (logflag.distpause)
		{
			longtimestamp();
			logflag.distpause=0;
		}
		write_word((LID_DISTANCE<<12)|(distdiff&0xFFF));
#if DEBUG
		fprintf(uart,"distdiff=%u\n",distdiff);
#endif			
	}else
		logflag.distpause=1;				
	
	if ( ! ( (systime/LOG_UNIT) & 0xff ) )
	{
		shorttimestamp();
	}
}
	

void longtimestamp(void)
{
	uint32_t longtime=(systime/LOG_UNIT)>>12;
	uint16_t buffer=LID_LONGTIMESTAMP<<12;
	buffer|= longtime&0xFFF;
	write_word(buffer);
#if DEBUG
	fprintf(uart,"longtimestamp=%lu\n",longtime);
#endif
}

	
void shorttimestamp(void)
{
	uint8_t shorttime=(systime/LOG_UNIT);
	uint16_t buffer=LID_SHORTTIMESTAMP<<12;
	buffer|=shorttime&0xFFF;
	write_word(buffer);
#if DEBUG
	fprintf(uart,"shorttimestamp=%u\n",shorttime);
#endif
}
	







