/* main.c
 */
/**
 * \page mainpage
 *
 * Problem:
 *   wir haben nur den Timer2 mit Anschluss an den 32kHz-Quarz.
 *   Der hat nur 8 bit. D.h. mit Prescaler 128. Haben wir eine Zeitauflsung von 1/256 sec.
 *   -> Das ist vielleicht ein bichen ungenau fr die Periodendauermessung.
 *         - der einzige Nachteil ist eine hhere Ungenauigkeit bei grossen Geschwindigkeiten.
 *   => Wir knnten den Prescaler verkleinern. (Stromverbrauch?)
 *   ==> Wir knnten die Peridendauermessung auf mehrere Radumdrehungen ausdehnen. (aufwendige SW?)
 * Nachteil am Timer2 ist auch, da es kein HW-Input Capture gibt.
 * 
 * ===> Timer2 als Zeitmessung mit 32er Prescaler!
 *       - damit knnen wir auch die Tasturentprellung ber systime machen.
 * 
 * Warum muss das LCD im Interrupthandler upgedated werden?
 * - Wohl nur wg. dem Scrolly-Zeugs.
 * -- Das Scrollzeugs brauche ich nicht. Textanzeige bis zu sechs Stellen ist ausreichend.
 * \todo SON-Rippleauswertung (schwimmender Analogkomparator?)
 * \todo warning messages
 *  \todo logging
 *  \todo Datenfilterung
 *  \todo Akkuspannungsberwachung
 *  \todo Lichtstromberwachung ?
 *  \section logging Data Logging
 *  Fr Rollversuche:
 *  NEMA-Output speichern (Pos, Altitude, Time)
 * \author Thomas Traber
 **/
/**
 * \page links Links
 * AVR222 Moving Average Filter
 * AVR223 Data Filters with AVR
 * AVR236 CRC Check of Program Memory
 * AVR350 Xmodem Receive Utility
 * AVR450 Battery Charger
 */

#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include <inttypes.h>
#include "main.h"
#include "settings.h"
#include "pins.h"
#include "ui.h"
#include "bf_lcd.h"
#include "dataflash.h"
#include "logging.h"


#define LOWBYTE(w)           ((uint8_t)(w))
#define HIBYTE(w)           ((uint8_t)(((uint16_t)(w) >> 8) & 0xFF))


uint16_t wheelcirc=256;		// in 1/256 Meter
volatile uint32_t systime;	// 1/4 Sekunden
volatile uint32_t ridingtime;	
volatile uint32_t totaldistance;// in 1/256 Meter
int temperature_current;
uint16_t inputvoltage;

/**
 * Today's Measurements 
 */
MARK daymark;

/**
 * Measurements since last break
 */
MARK breakmark;

MARK marks[NOOF_MARKS];

const char username_short[] PROGMEM = USERNAME_SHORT;
const char username_long[] PROGMEM = USERNAME_LONG;
const char bikename[] PROGMEM = BIKENAME;

volatile uint16_t pedaltime_current;	// 1/1024 Sekunden
volatile uint16_t wheeltime_current;     // 1/1024 Sekunden
volatile uint8_t pedal_active;
volatile uint8_t wheel_active;
volatile uint8_t system_active;
volatile uint16_t temp_counter;
volatile uint16_t volt_counter;

FILE * uart;

/**
 * \param Character to be sent via UART
 * \return 0
 */
int uart_putchar(char c)
{
	if (c == '\n')
		uart_putchar('\r');
	loop_until_bit_is_set(UCSRA,UDRE);
	UDR = c;
	return 0;
}

/**
 * \return Character from UART
 */
int uart_getchar(void)
{       
        loop_until_bit_is_set(UCSRA,RXC);
	 return UDR0;
}

/**
 * Initialisiation of RS232 Interface
 */
void uart_init(void)
{
	UBRRH = HIBYTE ( BAUD_DIV );
	UBRRL = LOWBYTE( BAUD_DIV );
	UCSRA = _BV(U2X);
	UCSRB = _BV(RXEN)|_BV(TXEN);
	UCSRC = (3<<UCSZ0);

}

/** 
 * Starts a single ADC Conversion on \em channel
 * \param channel 
 */
void trigger_adc(uint8_t channel)
{
	ADMUX=channel;
	ADCSRA=_BV(ADEN)|_BV(ADSC)|_BV(ADIF)|_BV(ADIE)|7;
}

/**
 * powers NTC and triggers a single ADC on channel 0
 */
void trigger_temp_meas(void)
{
	sbiBF(PORTF, PF3);	//switch on voltage at NTC	
	sbiBF(DDRF, DDF3);
	trigger_adc(0);
}

int main(void)
{    
				//\todo brauchen wir die Osz. Calibration berhaupt?
    //OSCCAL_calibration();     // calibrate the OSCCAL byte
        
    CLKPR = (1<<CLKPCE);        // set Clock Prescaler Change Enable

    CLKPR = (1<<CLKPS1) | (1<<CLKPS0);	// set prescaler = 8, Inter RC 8Mhz / 8 = 1Mhz

    ACSR = (1<<ACD);		// Disable Analog Comparator (power save)
    DIDR1 = (7<<ADC0D);		// Disable Digital input on PF0-2 (power save)
    PORTB = (15<<PB0);     	// Enable pullup on 
    PORTE = (15<<PE4);

    sbiBF(DDRB, 5);            	// set OC1A as output
    sbiBF(PORTB, 5);            // set OC1A high
            
    
    cli();

    cbiBF(TIMSK2, TOIE2);	// disable OCIE2A and TOIE2

    ASSR = (1<<AS2);		// select asynchronous operation of Timer2

    TCNT2 = 0;			// clear TCNT2A
    TCCR2A |= (1<<CS21) | (1<<CS20);             // precaler: 32.768 kHz / 32 / 256 = 1/4 sec between each overflow

    while((ASSR & 0x01) | (ASSR & 0x04))
	    ;       		// wait for TCN2UB and TCR2UB to be cleared

    TIFR2 = 0xFF;		// clear interrupt-flags
    sbiBF(TIMSK2, TOIE2);	// enable Timer2 overflow interrupt

    uart_init();                // Baud rate = 9600bps
    
    PCMSK0 |= _BV(WHEELINT_PIN)|_BV(PEDALINT_PIN);
    sei();
    df_spi_init();
    df_cs_inactive;        
    ui_init();			
    uart=fdevopen(uart_putchar,uart_getchar,0);
    puts_P(username_short);
    fputs_P(PSTR(PROGRAM_NAME " Version " PROGRAM_VERSION "\n"),uart);
    fputs_P(PSTR("Compiled at " __DATE__ "\n"),uart);
    fputs_P(PSTR("\nUsername:\t"),uart);
    fputs_P(username_long,uart);
    fputs_P(PSTR("\nBikename:\t"),uart);
    fputs_P(bikename,uart);
    fputs_P(PSTR("\nSystem Time:\t"),uart);
    print_time(uart,systime);
    //dataflashtest();
    menu();
}


/**
 * liest die Systime und die momentanen Zhlerstnde.
 * \return die Uhrzeit als Vielfaches von 1/1024 sec
 */
uint16_t getmstime()
{
		uint16_t mstime;
		cli();
		mstime=systime;
		mstime<<=8;
		mstime|=TCNT2;		//Zhler lesen
		sei();
		return mstime;
}

/**
 * \param ADC-Value
 * \return Temperatur in Degrees
 * grliche Rechnerei
 * wir sollten einen digitalen Temperatursensor einbauen
 */
int8_t calc_temp(uint16_t adc)
{
	//FIXME
	return adc>>8;
}


/**
 * \param ADC-Value
 * \return Voltage in 100mV Units
 * \todo So ein Mist, dass die Referenz vom LDR abhngt!
 * \todo Am Besten wir entfernen den LDR hw-mig!
 * Anstelle der LDR-Spannungsmessung wie im Butterfly Demo Code beschrieben,
 * sollte man die Spannung der Bandgap-Referenz zum Kalibrieren verwenden. 
 */
int8_t calc_volt(uint16_t adc)
{
	//FIXME
	return adc>>8;
}

/**
 * reads ADC Result 
 * switches ADC and NTC Power off
 */
SIGNAL(SIG_ADC)
{
	uint16_t result;
	
	result=ADCL;
	result|= (ADCH<<8);
	switch (ADMUX)
	{
		case TEMPERATURE_CHANNEL:
			temperature_current=calc_temp(result);
			break;
		case VOLTAGE_CHANNEL:
			inputvoltage=calc_volt(result);
			break;
	}
	cbiBF(PORTF,PF3);
	cbiBF(DDRF,DDF3);
	cbiBF(ADCSRA, ADEN);
}

/**
 * \page mainpage
 * \section rtcint RTC Interrupt
 * Im RTC-Interrupt Handler wird die System Time inkrementiert
 * Bei jedem Interruptaufruf wird "S3" getoggled.
 * \todo Display Update Call 
 * */
SIGNAL(SIG_OVERFLOW2)
{
		if (wheel_active)
		{
			wheel_active--;
			ridingtime++;
		}
		else
			wheeltime_current=0;
		
		if (pedal_active)
			pedal_active--;
		else
			pedaltime_current=0;
		
		if (system_active)
		{
			if (!wheel_active && !pedal_active)
				system_active--;
			
			if (!temp_counter)
			{
				temp_counter=TEMP_MEAS_INTERVAL;
				trigger_adc(TEMPERATURE_CHANNEL);		
			}
			else
				temp_counter--;

			if(!(systime & 0xF) )
				logupdate();	
		}
		
		if ( (systime&0x3) == 0 )		//jede Sekunde wird das Display upgedatet
			update_display();
		
#if LCD_AUTO_POWER_OFF
		if (!system_active)
			lcd_off();
#endif	
			
		systime++;
}


/**
 * \page mainpage
 * \section wheelint Rad Interrupt
 * Der Radinterrupt wird einmal pro Radumdrehung aufgerufen.
 * Es ist mit Frequenzen bis 20 Hz zu rechnen.
 * Im Zhler steht dann die Periodendauer.
 * Der Zhlerstand wird sw-mig ausgelesen. Der Zhler luft unendlich weiter. 
 * Bei jedem Interruptaufruf wird "S1" getoggled.
 * 
 * \todo knnen wir als Zhler den Timer2 der RTC mitbenutzen? Welcher Prescaler ist denn eingestellt? ca. 500 us Takt knnten wir uns schon erlauben.
 */
void wheelint(void)
{
	static uint16_t lasttime;
	uint16_t timediff;
	
	uint16_t time=getmstime();		
	timediff=time-lasttime;
	wheel_active=WHEEL_ACTIVE_LIMIT;
	
	if ( timediff < MINWHEELTIME ) 
		return;				//damit sollte das Prellen kein Problem mehr sein
#if 0  //DEBUG
	fprintf(uart,"Wint: time=%u\tlasttime=%u\ttimediff=%u\n",time,lasttime,timediff);
#endif
	*(pLCDREG+2)^=_BV(1);
	lasttime=time;
	totaldistance+=wheelcirc;
	wheeltime_current=timediff;
#if 0// DEBUG
	fprintf(uart,"Wint: totaldistance=%u\twheeltime_current=%u\n",totaldistance,wheeltime_current);
#endif	
}

/**
 * \page mainpage
 * \section pedalint Pedal Interrupt
 * Die Frequenz hier wird deutlich unter 10Hz bleiben. 
 * D.h. es ist wirklich nicht ntig, ein HW-Zhler-Latch zu verwenden.
 * Wir lesen einfach den gleichen Zhler wie beim Radint aus.
 * Bei jedem Interruptaufruf wird "S2" getoggled.
 */
void pedalint(void)
{
	static uint16_t lasttime;
	uint16_t timediff;
	uint16_t time=getmstime();		
	timediff=time-lasttime;
	pedal_active=PEDAL_ACTIVE_LIMIT;
	if ( timediff < MINPEDALTIME ) 
		return;				//damit sollte das Prellen kein Problem mehr sein
#if 0 //DEBUG
	fprintf(uart,"PedInt: time=%u\tlasttime=%u\ttimediff=%u\n",time,lasttime,timediff);
#endif
	*(pLCDREG+2)^=_BV(5);
	lasttime=time;
	pedaltime_current=timediff;
}


SIGNAL(SIG_PIN_CHANGE0)
{
	system_active=SYSTEM_ACTIVE_LIMIT;
#ifdef _AVR_IOM169_H_
	if (!(PINE&_BV(WHEELINT_PIN)))
		wheelint();
	else if (!(PINE&_BV(PEDALINT_PIN)))
		pedalint();
	else
#endif		
		joystick_interrupt();
}

SIGNAL(SIG_PIN_CHANGE1)
{
	joystick_interrupt();
	system_active=SYSTEM_ACTIVE_LIMIT;
}

