/* ui.c
 */
/**
 * \section ui User Interface
 *
 * Aus dem RTC-Timer Interrupt wird update_display() aufgerufen, um die Anzeige upzudaten.
 * Was angezeigt wird, hngt vom ausgewhlten Menupunkt ab.
 */

#include <avr/io.h>
#include <stdlib.h>
#include <stdio.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <inttypes.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include "main.h"
#include "settings.h"
#include "bf_lcd.h"
#include "ui.h"

extern FILE * uart;

enum{ S_TIME, S_RTIME, S_BTIME, S_DTIME, S_SPEED, S_TOTDIST, S_PEDAL, S_TEMP, S_VOLTAGE, S_OPTION, S_END};

const char s_time[] 	PROGMEM ="Time  ";
const char s_rtime[] 	PROGMEM ="RideTm";
const char s_btime[] 	PROGMEM ="BrkTim";
const char s_dtime[] 	PROGMEM ="DayTim";
const char s_speed[] 	PROGMEM ="Speed ";
const char s_totdist[] 	PROGMEM ="TlDist";
const char s_pedal[] 	PROGMEM ="PedalF";
const char s_temp[] 	PROGMEM ="Temp  ";
const char s_voltage[] 	PROGMEM ="Voltge";
const char s_option[]	PROGMEM ="Option";

PGM_P selectionname[S_END] PROGMEM =
{
	s_time,
	s_rtime,
	s_btime,
	s_dtime,
	s_speed,
	s_totdist,
	s_pedal,
	s_temp,
	s_voltage,
	s_option
};


enum{O_LOGBIN,O_LOGHEX,O_LOGASC,O_LOGDEL,O_MEMUSED,O_END};

const char o_logbin[] PROGMEM = "LogBin";
const char o_loghex[] PROGMEM = "LogHex";
const char o_logasc[] PROGMEM = "LogAsc";
const char o_logdel[] PROGMEM = "DelLog";
const char o_memused[] PROGMEM = "MemUse";

PGM_P optionname[O_END] PROGMEM = 
{
	o_logbin,
	o_loghex,
	o_logasc,
	o_logdel,
	o_memused
};

extern volatile uint32_t systime;
extern volatile uint32_t wheeltime_current;
extern volatile uint32_t pedaltime_current;
extern volatile uint8_t pedal_active;
extern volatile uint8_t wheel_acitve;
extern uint16_t wheelcirc;
extern uint32_t lastbreaktime;
extern volatile uint32_t ridingtime;
extern uint16_t inputvoltage;
extern int temperature_current;
extern uint32_t totaldistance;

extern FILE *uart;
extern MARK daymark;
extern MARK breakmark;
extern MARK marks[NOOF_MARKS];

volatile uint8_t selection;
volatile char key;
volatile uint8_t display_lock=0;

/**
 * \param time
 * printet die Differenzzeit
 * Um die aktuell Uhrzeit anzuzeigen, muss print_time(systime) aufgerufen werden.
 */
void print_time(FILE * file, uint32_t time)
{
	ldiv_t min;
	ldiv_t hour;
	ldiv_t day;
	uint32_t secs=time/4;
	min=ldiv(secs,60);
	hour=ldiv(min.quot,60);
	day=ldiv(hour.quot,24);
	fprintf_P(file,PSTR("%.2ld:%.2ld:%.2ld\n"),day.rem,hour.rem,min.rem);
	#if DEBUG 
	fprintf_P(uart,PSTR("%.2ld:%.2ld:%.2ld\n"),day.rem,hour.rem,min.rem);
	fprintf_P(uart,PSTR("%i\n"),secs);
	#endif
}

/**
 * \param distance 
 */
void print_distance(uint32_t dist)
{
	printf("%.6ld\n", (uint32_t) dist/256/100);
}

/**
 * \param wheeltime
 * speed=(wheelcirc/256)/(wheeltime/1024)*3.6 km/h   
 */
void print_speed(uint16_t wheeltime)
{
	uint32_t speed;
	if (wheeltime)
	{
		speed=(uint32_t)wheelcirc*360;  	//0.01 km/h display resolution
		fprintf(uart,"wheelcirc*360*256 %lu\n",speed);
		speed*=4;
		fprintf(uart,"speed*1024 %lu\n",speed);
		speed/=wheeltime;
	}
	else
		speed=0;
	printf("%.4lukm:\n",speed);
#if DEBUG
	fprintf(uart,"%4ludm\t%u\n",speed,wheeltime);
#endif
}

/**
 * \param pedaltime
 * pedalfreq=60/pedaltime
 * 1/10 Pedalumdrehungen/Minute
 * \todo Runden
 */
void print_pedalfreq(uint16_t pedaltime)
{
	uint16_t pedalfreq;
		
	if(pedaltime)
		pedalfreq= (uint16_t) 60*1024/pedaltime;
	else
		pedalfreq=0;

	printf_P(PSTR("%6u\n"),pedalfreq);
	
}

void print_temperature(int temp)
{
	printf_P(PSTR("%4iC\n"),temp);
}

void print_voltage(uint16_t voltage)
{
	printf_P(PSTR("%5uV\n"),voltage);
}

/**
 * wird normalerweise vom RTC-Interrupt aus jede Sekunde aufgerufen
 */
void update_display()
{
	if (display_lock)
	{
		if (display_lock>0)
			display_lock--;
		return;
	}
	
	switch(selection)
	{
		case S_TIME:	print_time(stdout,systime);break;
		case S_BTIME:	print_time(stdout,systime-breakmark.starttime);break;
		case S_DTIME:	print_time(stdout,systime-daymark.starttime);break;
		case S_RTIME:	print_time(stdout,ridingtime);break;
		case S_SPEED:	print_speed(wheeltime_current);break;
		case S_TOTDIST: print_distance(totaldistance);break;
		case S_PEDAL:   print_pedalfreq(pedaltime_current);break;
		case S_TEMP:	print_temperature(temperature_current);break;
		case S_VOLTAGE:	print_voltage(inputvoltage);break;
	}
}


void joystick_interrupt(void)
{
	static uint16_t lasttime;
	uint16_t keytime;
	char inkey;
	inkey = (~PINB) & PINB_MASK;
	inkey |= (~PINE) & PINE_MASK;
	EIFR = (1<<PCIF1) | (1<<PCIF0);
	
	
	keytime=getmstime();
	
	if ( (keytime-lasttime) < KEYDELAY) return;
	
	lasttime=keytime;
	key=inkey;
}

/**
 * \return "Keyboard" Scan Code
 */
int getkey(void)
{
	int retkey;
	while(key==0)
	{
		set_sleep_mode(SLEEP_MODE_PWR_SAVE);
		sleep_mode();
	}
	retkey=key;
	key=0;
	return retkey;
}

/**
 * \return Character 
 * \todo Tastaturmenu fr den Butterfly
 */
int get_char(void)
{
	int c;
	c=getkey();
	return c;
}	
			

void joystick_init(void)
{
	cbiBF(DDRB,7);
	cbiBF(DDRB,6);
	cbiBF(DDRB,4);
	PORTB |= PINB_MASK;
	DDRE = 0x00;
	PORTE |= PINE_MASK;
	// Enable pin change interrupt on PORTB and PORTE
	PCMSK0 |= PINE_MASK;
	PCMSK1 |= PINB_MASK;
	EIFR = (1<<PCIF0)|(1<<PCIF1);
	EIMSK = (1<<PCIE0)|(1<<PCIE1);
}

void ui_init(void)
{
	joystick_init();
	lcd_init();
	fdevopen(lcd_writechar,get_char,0);
	puts_P(PSTR("InitOK"));
}

void delete_log(void)
{
	char answ='n';
	
	puts_P(PSTR("Del?!?\n"));
	while (answ!='q')
	{
		switch (getkey())
		{
			case KEY_PUSH:
				if (answ=='y')
				{
					log_delete();
				}
				answ='q';
				break;
			case KEY_UP:
			case KEY_DOWN:
				if (answ=='n')
				{
					answ='y';
					puts_P(PSTR("  YES \n"));
				}else
				{
					answ='n';
					puts_P(PSTR("  NO  \n"));
				}
				break;
			case KEY_LEFT:
				answ='q';
				break;
		}
	}		
}

void optionfunc(option)
{
	switch(option)
	{
		case O_LOGBIN:
			log_binary_output();
			break;
		case O_LOGASC:
			log_ascii_output();
			break;
		case O_LOGHEX:
			log_hex_output();
			break;
		case O_LOGDEL:
			delete_log();
			break;
		case O_MEMUSED:
			printf("used%2d\n",log_mem_used());
			getkey();
			break;
	}
}

void optionmenu()
{
	uint8_t end;
	uint8_t option;
	int key;
	end=0;
	option=0;
	display_lock=-1;			//no display update in RTC-Interrupt
	#if DEBUG
	fprintf(uart,"%s option=%i %i\n",__FUNCTION__,option,end);
	#endif
	while(!end)
	{
		puts_P(pgm_read_word_near(&optionname[option])); 
		key=getkey();
		switch(key)
		{
			case KEY_UP:
				if (option < (O_END-1))
					option++;
				else
					option=0;
				break;
			case KEY_DOWN:
				if (option)
					option--;
				else
					option=O_END-1;
				break;
			case KEY_PUSH:
				optionfunc(option);
				end=1;
				break;
			case KEY_LEFT:
				end=1;
				break;
		}
	}
}

void submenu()
{
 	#if DEBUG
	fprintf(uart,"%s selection=%i\n",__FUNCTION__,selection);
	#endif
	switch(selection)
	{
		case S_OPTION:
			optionmenu();
			break;
	}
}

/**
 * 
 */
void menu(void)
{
	int key;
	while(1)
	{
		key=getkey();
		switch(key)
		{
			case KEY_UP: 	
				if (selection < (S_END-1))
					selection++;
				else
					selection=0;
				break;
			case KEY_DOWN: 	
				if (selection)
					selection--;
				else
					selection=S_END-1;
				break;
			case KEY_PUSH:	
				display_lock=0;	
				continue;	
			case KEY_RIGHT:
				submenu();
				break;
		}
		puts_P(pgm_read_word_near(&selectionname[selection])); 
		display_lock=2;
	}
}


